package org.processmining.xeslite.plugin;

import java.io.InputStream;

import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.xeslite.external.MapDBStore;
import org.processmining.xeslite.external.XFactoryExternalStore;

/**
 * Open XES log using a buffered implementation {@link XFactoryExternalStore}
 * for the attributes of all XES objects. In this implementation all attributes
 * are immutable. Therefore, it does not support the MXML parser, which tries to
 * change the values of attributes after creation.
 * 
 * @author F. Mannhardt
 * 
 */
@Plugin(name = "Open XES Log File (Disk-buffered by MapDB, sequential access, w/o cache)", //
level = PluginLevel.Regular, parameterLabels = { "Filename" }, returnLabels = { "Log (single process)" }, returnTypes = { XLog.class })
@UIImportPlugin(description = "ProM log files (Disk-buffered by MapDB, sequential access, w/o cache)", //
extensions = { "gz", "zip", "xes", "xez" })
// does not support MXML parser
public class OpenLogFileDiskSequentialAccessImplWithoutCachePlugin extends AbstractOpenLogFileLitePlugin {

	@Override
	protected Object importFromStream(PluginContext context, InputStream input, String filename, long fileSizeInBytes)
			throws Exception {
		MapDBStore.Builder builder = new MapDBStore.Builder().withPump();
		XFactoryExternalStore factory = new XFactoryExternalStore.MapDBDiskSequentialAccessWithoutCacheImpl(builder);
		return importFromUsingFactory(context, input, filename, fileSizeInBytes, factory);
	}

	protected void beforeParse(XFactory factory) {
		((XFactoryExternalStore) factory).startPump();		
	}

	protected void afterParse(XFactory factory) {
		try {
			((XFactoryExternalStore) factory).finishPump();
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
			return;
		}		
	}
	
}
